@@ -12,17 +12,16 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#ifndef LOBSTER_ENGINE
-#define LOBSTER_ENGINE
+#ifndef LOBSTER_ENGINE_H
+#define LOBSTER_ENGINE_H
 
-#include "lobster/natreg.h"
+#include "lobster/vmdata.h"
 
 extern void RegisterCoreEngineBuiltins(lobster::NativeRegistry &natreg);
-extern void EngineRunByteCode(lobster::NativeRegistry &natreg, const char *fn, string &bytecode,
-                              const void *entry_point, const void *static_bytecode,
-                              const vector<string> &program_args);
-extern int EngineRunCompiledCodeMain(int argc, char *argv[], const void *entry_point,
-                                     const void *bytecodefb);
+extern void EngineRunByteCode(lobster::VMArgs &&vmargs);
+extern "C" int EngineRunCompiledCodeMain(int argc, char *argv[], const void *entry_point,
+                                         const void *bytecodefb, size_t static_size,
+                                         const lobster::block_t *vtables);
 extern void EngineSuspendIfNeeded();
 extern void EngineExit(int code);
 
@@ -30,4 +29,4 @@ extern void EngineExit(int code);
 #define USE_MAIN_LOOP_CALLBACK
 #endif
 
-#endif  // LOBSTER_ENGINE
+#endif  // LOBSTER_ENGINE_H
