@@ -1,3 +1,19 @@
+// Copyright 2014 Wouter van Oortmerssen. All rights reserved.
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+
+#ifndef LOBSTER_MESHGEN_H
+#define LOBSTER_MESHGEN_H
 
 #include "3dgrid.h"
 
@@ -31,3 +47,5 @@ inline void RecomputeNormals(vector<int> &triangles, vector<mgvert> &verts) {
 
 extern Mesh *polygonize_mc(const int3 &gridsize, float gridscale, const float3 &gridtrans,
                            const DistGrid *distgrid, float3 (* grid_to_world)(const int3 &pos));
+
+#endif  // LOBSTER_MESHGEN_H
