@@ -140,7 +140,7 @@ struct Document {
     void InitWith(Cell *r, wxString filename) {
         rootgrid = r;
         selected = Selection(r->grid, 0, 0, 1, 1);
-        ChangeFileName(filename);
+        ChangeFileName(filename, false);
     }
 
     void UpdateFileName(int page = -1) {
@@ -148,8 +148,12 @@ struct Document {
                                  page);
     }
 
-    void ChangeFileName(const wxString &fn) {
+    void ChangeFileName(const wxString &fn, bool checkext) {
         filename = fn;
+        if (checkext) {
+            wxFileName wxfn(filename);
+            if (!wxfn.HasExt()) filename.Append(L".cts");
+        }
         UpdateFileName();
     }
 
@@ -264,13 +268,18 @@ struct Document {
 
     void ScrollOrZoom(wxDC &dc, bool zoomiftiny = false) {
         if (!selected.g) return;
-        for (Cell *cg = selected.g->cell; cg; cg = cg->parent)
+        Cell *drawroot = WalkPath(drawpath);
+        // If we jumped to a cell which may be insided a folded cell, we have to unfold it
+        // because the rest of the code doesn't deal with a selection that is invisible :)
+        for (Cell *cg = selected.g->cell; cg; cg = cg->parent) {
+            // Unless we're under the drawroot, no need to unfold further.
+            if (cg == drawroot) break;
             if (cg->grid->folded) {
                 cg->grid->folded = false;
                 cg->ResetLayout();
                 cg->ResetChildren();
             }
-        Cell *drawroot = WalkPath(drawpath);
+        }
         for (Cell *cg = selected.g->cell; cg; cg = cg->parent)
             if (cg == drawroot) {
                 if (zoomiftiny) ZoomTiny(dc);
@@ -313,8 +322,9 @@ struct Document {
         ResetFont();
         Selection prev = hover;
         hover = Selection();
-        WalkPath(drawpath)->grid->FindXY(this, x - centerx / currentviewscale - hierarchysize,
-                                         y - centery / currentviewscale - hierarchysize, dc);
+        auto drawroot = WalkPath(drawpath);
+        if (drawroot->grid) drawroot->grid->FindXY(this, x - centerx / currentviewscale - hierarchysize,
+                                                   y - centery / currentviewscale - hierarchysize, dc);
         if (!(prev == hover)) {
             if (prev.g) prev.g->DrawHover(this, dc, prev);
             if (hover.g) hover.g->DrawHover(this, dc, hover);
@@ -343,13 +353,16 @@ struct Document {
         Cell *tc = begindrag.ThinExpand(this);
         selected = begindrag;
         if (tc) {
-            tc->Paste(this, c->Clone(nullptr), begindrag);
-            if (isctrlshiftdrag == 1) {
+            auto is_parent = tc->IsParentOf(c);
+            auto tc_parent = tc->parent;  // tc may be deleted.
+            tc->Paste(this, c, begindrag);
+            // If is_parent, c has been deleted already.
+            if (isctrlshiftdrag == 1 && !is_parent) {
                 c->parent->AddUndo(this);
                 Selection cs = c->parent->grid->FindCell(c);
                 c->parent->grid->MultiCellDeleteSub(this, cs);
             }
-            hover = selected = tc->parent->grid->FindCell(tc);
+            hover = selected = tc_parent ? tc_parent->grid->FindCell(tc) : Selection();
         }
         Refresh();
     }
@@ -387,12 +400,12 @@ struct Document {
             CreatePath(c && c->grid ? c : selected.g->cell, drawpath);
         } else if (dir < 0) {
             Cell *drawroot = WalkPath(drawpath);
-            if (drawroot->grid->folded && selectionmaybedrawroot)
+            if (drawroot->grid && drawroot->grid->folded && selectionmaybedrawroot)
                 selected = drawroot->parent->grid->FindCell(drawroot);
         }
         while (len < drawpath.size()) drawpath.remove(0);
         Cell *drawroot = WalkPath(drawpath);
-        if (selected.GetCell() == drawroot) {
+        if (selected.GetCell() == drawroot && drawroot->grid) {
             selected = Selection(drawroot->grid, 0, 0, drawroot->grid->xs, drawroot->grid->ys);
         }
         drawroot->ResetLayout();
@@ -557,13 +570,14 @@ struct Document {
     bool PickFont(wxDC &dc, int depth, int relsize, int stylebits) {
         int textsize = TextSize(depth, relsize);
         if (textsize != lasttextsize || stylebits != laststylebits) {
-            dc.SetFont(
-                wxFont(textsize - (while_printing || scaledviewingmode),
-                       stylebits & STYLE_FIXED ? wxFONTFAMILY_TELETYPE : wxFONTFAMILY_DEFAULT,
-                       stylebits & STYLE_ITALIC ? wxFONTSTYLE_ITALIC : wxFONTSTYLE_NORMAL,
-                       stylebits & STYLE_BOLD ? wxFONTWEIGHT_BOLD : wxFONTWEIGHT_NORMAL,
-                       (stylebits & STYLE_UNDERLINE) != 0,
-                       stylebits & STYLE_FIXED ? L"" : sys->defaultfont));
+            wxFont font(textsize - (while_printing || scaledviewingmode),
+                   stylebits & STYLE_FIXED ? wxFONTFAMILY_TELETYPE : wxFONTFAMILY_DEFAULT,
+                   stylebits & STYLE_ITALIC ? wxFONTSTYLE_ITALIC : wxFONTSTYLE_NORMAL,
+                   stylebits & STYLE_BOLD ? wxFONTWEIGHT_BOLD : wxFONTWEIGHT_NORMAL,
+                   (stylebits & STYLE_UNDERLINE) != 0,
+                   stylebits & STYLE_FIXED ? L"" : sys->defaultfont);
+            if (stylebits & STYLE_STRIKETHRU) font.SetStrikethrough(true);
+            dc.SetFont(font);
             lasttextsize = textsize;
             laststylebits = stylebits;
         }
@@ -717,7 +731,7 @@ struct Document {
                              L"TreeSheets Files (*.cts)|*.cts|All Files (*.*)|*.*",
                              wxFD_SAVE | wxFD_OVERWRITE_PROMPT | wxFD_CHANGE_DIR);
         if (fn.empty()) return _(L"Save cancelled.");  // avoid name being set to ""
-        ChangeFileName(fn);
+        ChangeFileName(fn, true);
         return SaveDB(success);
     }
 
@@ -1222,11 +1236,10 @@ struct Document {
                 if (selected.TextEdit()) break;
                 if (selected.g->cell->parent) {
                     selected = selected.g->cell->parent->grid->FindCell(selected.g->cell);
-                    ScrollOrZoom(dc);
                 } else {
                     selected.SelAll();
-                    Refresh();
                 }
+                ScrollOrZoom(dc);
                 return nullptr;
 
             case A_NEWGRID:
@@ -1238,13 +1251,12 @@ struct Document {
                     c->AddUndo(this);
                     c->AddGrid();
                     selected = Selection(c->grid, 0, 0, 1, 1);
-                    Refresh();
+                    DrawSelectMove(dc, selected, true);
                 }
                 return nullptr;
 
             case A_PASTE:
                 if (!(c = selected.ThinExpand(this))) return OneCell();
-
                 if (wxTheClipboard->Open()) {
                     wxTheClipboard->GetData(*dataobjc);
                     PasteOrDrop();
@@ -1269,7 +1281,7 @@ struct Document {
                     selected.Cursor(this, A_DOWN, false, false, dc, true);
                 } else {
                     selected.EnterEdit(this, 0, (int)c->text.t.Len());
-                    Refresh();
+                    DrawSelectMove(dc, selected, true);
                 }
                 return nullptr;
 
@@ -1778,7 +1790,7 @@ struct Document {
         if (beforesel.g) CreatePath(beforesel.g->cell, beforepath);
         UndoItem *ui = fromlist.pop();
         Cell *c = WalkPath(ui->path);
-        if (c->parent) {
+        if (c->parent && c->parent->grid) {
             c->parent->grid->ReplaceCell(c, ui->clone);
             ui->clone->parent = c->parent;
         } else
