@@ -69,6 +69,9 @@ WX_DECLARE_STRING_HASH_MAP(bool, wxHashMapBool);
 #include <new>
 
 #include <vector>
+#include <string>
+#include <string_view>
+#include <map>
 
 #include <utility>
 
